// Dimensions for each chart
const margin = { top: 30, right: 20, bottom: 60, left: 40 },
  width = 220 - margin.left - margin.right,
  height = 140 - margin.top - margin.bottom;

// Read the csv file
d3.csv(
  'https://gist.githubusercontent.com/riyamathur1/9e6a69695dce47b51253ae50fdf87783/raw/637686002b0c59b5320a8506e4c4b199abde1581/atlanticstorms.csv',
).then((data) => {
  // Parsing year, converting from string -> num
  const parseYear = d3.timeParse('%Y');
  data.forEach((d) => {
    d.frequency = +d.frequency;
    d.year = parseYear(d.year);
  });

  // Grouping data by storm type
  const dataNest = Array.from(
    d3.group(data, (d) => d.name),
  );

  // X and Y axis scales
  const xScale = d3
    .scaleTime()
    .domain(d3.extent(data, (d) => d.year))
    .range([0, width]);
  const yScale = d3
    .scaleLinear()
    .domain([0, d3.max(data, (d) => d.frequency)])
    .range([height, 0]);

  // Generating lines
  const line = d3
    .line()
    .x((d) => xScale(d.year))
    .y((d) => yScale(d.frequency));

  // The color scale
  const color = d3.scaleOrdinal(d3.schemeCategory10);

  // SVG object for each small multiple
  const charts = d3
    .select('#small-multiples')
    .selectAll('svg')
    .data(dataNest)
    .enter()
    .append('svg')
    .attr('class', 'chart')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Drawing axes and append line paths to each chart
  charts.each(function (d, i) {
    const chart = d3.select(this);

    // Adding X and Y axes for each chart
    chart
      .append('g')
      .attr('class', 'axis x-axis')
      .attr('transform', `translate(0,${height})`)
      .call(d3.axisBottom(xScale).ticks(5));

    chart
      .append('g')
      .attr('class', 'axis y-axis')
      .call(d3.axisLeft(yScale).ticks(3));

    // Adding the line path for the specific storm type
    chart
      .append('path')
      .datum(d[1]) // To bind the specific data for each storm type
      .attr('fill', 'none')
      .attr('stroke', color(d[0]))
      .attr('stroke-width', 2)
      .attr('d', line);

    // Adding labels for the y-axis on the left side of each chart
    chart
      .append('text')
      .attr('transform', 'rotate(-90)')
      .attr('y', 0 - margin.left)
      .attr('x', 0 - height / 2)
      .attr('dy', '1em')
      .style('text-anchor', 'middle')
      .text('Frequency');

    // Adding labels for the x-axis on the bottom of each chart
    chart
      .append('text')
      .attr(
        'transform',
        `translate(${width / 2}, ${height + margin.bottom / 1.5})`,
      )
      .style('text-anchor', 'middle')
      .text('Year');

    // Titles for each chart
    chart
      .append('text')
      .attr('x', width / 2)
      .attr('y', 0 - margin.top / 2)
      .attr('text-anchor', 'middle')
      .style('font-size', '16px')
      .style('font-weight', 'bold')
      .text(d[0]);
  });

  // Legend placed at the bottom
  const legend = d3
    .select('#small-multiples')
    .append('div')
    .attr('class', 'legend');

  dataNest.forEach((d, i) => {
    const legendItem = legend
      .append('div')
      .attr('class', 'legend-item');

    legendItem
      .append('div')
      .attr('class', 'legend-color')
      .style('background-color', color(d[0]));

    legendItem.append('text').text(d[0]);
  });
});
